"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var vscode_languageserver_1 = require("vscode-languageserver");
var builtin_1 = require("../server/builtin");
var kinds_1 = require("../server/kinds");
var util_1 = require("../util/util");
var logger_1 = tslib_1.__importDefault(require("../server/logger"));
var snippets_1 = require("../server/snippets");
var log = logger_1.default('completion');
/*
 * predefined vim variable: v:xxx
 */
function predefinedVimVariablesProvider(line) {
    if (util_1.isSomeMatchPattern(kinds_1.builtinVariablePattern, line)) {
        return builtin_1.server.getPredefinedVimVariables();
    }
    return [];
}
/*
 * vim options: &xxx
 */
function vimOptionsProvider(line) {
    if (util_1.isSomeMatchPattern(kinds_1.optionPattern, line)) {
        return builtin_1.server.getVimOptions();
    }
    return [];
}
// vim builtin functions
function builtinVimFunctionProvider(line) {
    if (util_1.isSomeMatchPattern(kinds_1.notBuiltinFunctionPattern, line)) {
        return [];
    }
    return builtin_1.server.getBuiltinVimFunctions();
}
/*
 * vim commands
 */
function vimCommandsProvider(line) {
    if (util_1.isSomeMatchPattern(kinds_1.commandPattern, line)) {
        return builtin_1.server.getVimCommands().concat(snippets_1.commandSnippets);
    }
    return [];
}
/*
 * has features
 */
function vimFeaturesProvider(line) {
    if (util_1.isSomeMatchPattern(kinds_1.featurePattern, line)) {
        return builtin_1.server.getVimFeatures();
    }
    return [];
}
/*
 * expand():
 *   ['<cfile>', '<afile>', '<abuf>', '<amatch>', '<sfile>', '<cword>', '<cWORD>', '<client>']
 */
function expandKeywordsProvider(line) {
    if (kinds_1.expandPattern[0].test(line)) {
        return builtin_1.server.getExpandKeywords().map(function (item) {
            return tslib_1.__assign({}, item, { insertText: item.insertText.slice(1) });
        });
    }
    else if (kinds_1.expandPattern[1].test(line)) {
        return builtin_1.server.getExpandKeywords();
    }
    return [];
}
/*
 * color schemes
 */
function colorschemesProvider(line) {
    if (kinds_1.colorschemePattern.test(line)) {
        return builtin_1.server.getColorschemes();
    }
    return [];
}
/*
 * map command args
 */
function mapArgsProvoder(line) {
    if (kinds_1.mapCommandPattern.test(line)) {
        if (/<$/.test(line)) {
            return builtin_1.server.getVimMapArgs().map(function (item) { return (tslib_1.__assign({}, item, { insertText: item.insertText.slice(1) })); });
        }
        return builtin_1.server.getVimMapArgs();
    }
    return [];
}
function getCompletion(documents, params) {
    var textDocument = params.textDocument, position = params.position;
    var textDoc = documents.get(textDocument.uri);
    if (textDoc) {
        var line = textDoc.getText(vscode_languageserver_1.Range.create(vscode_languageserver_1.Position.create(position.line, 0), position));
        return [].concat(predefinedVimVariablesProvider(line)).concat(vimOptionsProvider(line)).concat(builtinVimFunctionProvider(line)).concat(expandKeywordsProvider(line)).concat(vimCommandsProvider(line)).concat(vimFeaturesProvider(line)).concat(colorschemesProvider(line)).concat(mapArgsProvoder(line));
    }
    return [];
}
exports.getCompletion = getCompletion;
//# sourceMappingURL=completion.js.map