"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var parser_1 = require("./server/parser");
var completion_1 = require("./handles/completion");
var builtin_1 = require("./server/builtin");
var hover_1 = require("./handles/hover");
var completionResolve_1 = require("./handles/completionResolve");
var signatureHelp_1 = require("./handles/signatureHelp");
var documents_1 = require("./server/documents");
var connection_1 = require("./server/connection");
// lsp initialize
connection_1.connection.onInitialize(function (param) {
    var _a = param.initializationOptions, initializationOptions = _a === void 0 ? {} : _a;
    var runtimepath = initializationOptions.runtimepath, vimruntime = initializationOptions.vimruntime;
    builtin_1.server.init({
        runtimepath: runtimepath ? runtimepath.split(',') : [],
        vimruntime: vimruntime || ''
    });
    return {
        capabilities: {
            textDocumentSync: documents_1.documents.syncKind,
            hoverProvider: true,
            completionProvider: {
                triggerCharacters: ['.', ':', '#', '[', '&', '$', '<', '"', "'"],
                resolveProvider: true
            },
            signatureHelpProvider: {
                triggerCharacters: ['(', ',']
            },
        }
    };
});
// document change or open
documents_1.documents.onDidChangeContent(function (change) {
    parser_1.next(change.document, connection_1.connection);
});
documents_1.documents.onDidClose(function (evt) {
    parser_1.unsubscribe(evt.document);
});
// listen for document's open/close/change
documents_1.documents.listen(connection_1.connection);
// handle completion
connection_1.connection.onCompletion(function (params) {
    return completion_1.getCompletion(documents_1.documents, params);
});
// handle completion resolve
connection_1.connection.onCompletionResolve(function (params) {
    return completionResolve_1.getCompletionResolve(params);
});
connection_1.connection.onSignatureHelp(function (params) {
    return signatureHelp_1.getSignatureHelp(params, documents_1.documents);
});
// handle hover
connection_1.connection.onHover(function (params) {
    return hover_1.getHover(params, documents_1.documents);
});
// lsp start
connection_1.connection.listen();
//# sourceMappingURL=index.js.map