"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs_1 = tslib_1.__importDefault(require("fs"));
var path_1 = tslib_1.__importDefault(require("path"));
var findup_1 = tslib_1.__importDefault(require("findup"));
var child_process_1 = require("child_process");
function isSomeMatchPattern(patterns, line) {
    return patterns.some(function (p) { return p.test(line); });
}
exports.isSomeMatchPattern = isSomeMatchPattern;
function executeFile(input, command, args, option) {
    return new Promise(function (resolve, reject) {
        var stdout = '';
        var stderr = '';
        var error;
        var isPassAsText = false;
        args = (args || []).map(function (arg) {
            if (/%text/.test(arg)) {
                isPassAsText = true;
                return arg.replace(/%text/g, input.toString());
            }
            return arg;
        });
        var cp = child_process_1.spawn(command, args, option);
        cp.stdout.on('data', function (data) {
            stdout += data;
        });
        cp.stderr.on('data', function (data) {
            stderr += data;
        });
        cp.on('error', function (err) {
            error = err;
            reject(error);
        });
        cp.on('close', function (code) {
            if (!error) {
                resolve({ code: code, stdout: stdout, stderr: stderr });
            }
        });
        // error will occur when cp get error
        if (!isPassAsText) {
            input.pipe(cp.stdin).on('error', function () { });
        }
    });
}
exports.executeFile = executeFile;
// cover cb type async function to promise
function pcb(cb) {
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return new Promise(function (resolve) {
            cb.apply(void 0, args.concat([function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    resolve(args);
                }]));
        });
    };
}
exports.pcb = pcb;
// find work dirname by root patterns
function findWorkDirectory(filePath, rootPatterns) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var dirname, patterns, _i, patterns_1, pattern, _a, err, dir;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    dirname = path_1.default.dirname(filePath);
                    patterns = [].concat(rootPatterns);
                    _i = 0, patterns_1 = patterns;
                    _b.label = 1;
                case 1:
                    if (!(_i < patterns_1.length)) return [3 /*break*/, 4];
                    pattern = patterns_1[_i];
                    return [4 /*yield*/, pcb(findup_1.default)(dirname, pattern)];
                case 2:
                    _a = _b.sent(), err = _a[0], dir = _a[1];
                    if (!err && dir && dir !== '/') {
                        return [2 /*return*/, dir];
                    }
                    _b.label = 3;
                case 3:
                    _i++;
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/, dirname];
            }
        });
    });
}
exports.findWorkDirectory = findWorkDirectory;
function findCommand(command, workDir) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var cmd;
        return tslib_1.__generator(this, function (_a) {
            if (/^(\.\.|\.)/.test(command)) {
                cmd = path_1.default.join(workDir, command);
                if (fs_1.default.existsSync(cmd)) {
                    return [2 /*return*/, command];
                }
                return [2 /*return*/, path_1.default.basename(cmd)];
            }
            return [2 /*return*/, command];
        });
    });
}
exports.findCommand = findCommand;
function markupSnippets(snippets) {
    return [
        '``` vim',
        snippets.replace(/\$\{[0-9]+(:([^}]+))?\}/g, '$2'),
        '```'
    ].join('\n');
}
exports.markupSnippets = markupSnippets;
//# sourceMappingURL=util.js.map